//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Core/Background/PoissonNoiseBackground.h
//! @brief     Defines class PoissonNoiseBackground.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_CORE_BACKGROUND_POISSONNOISEBACKGROUND_H
#define BORNAGAIN_CORE_BACKGROUND_POISSONNOISEBACKGROUND_H

#include "Core/Background/IBackground.h"

//! Class representing Poisson noise on top of the scattered intensity
//!
//! @ingroup simulation

class PoissonNoiseBackground : public IBackground {
public:
    inline static const std::string class_name = "PoissonNoiseBackground";
    std::string className() const final { return class_name; }

    PoissonNoiseBackground();
    PoissonNoiseBackground* clone() const override;

    double addBackground(double intensity) const override;
};

#endif // BORNAGAIN_CORE_BACKGROUND_POISSONNOISEBACKGROUND_H
