//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/From/GUISampleBuilder.h
//! @brief     Defines class GUISampleBuilder
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_FROM_GUISAMPLEBUILDER_H
#define BORNAGAIN_GUI_MODEL_FROM_GUISAMPLEBUILDER_H

#include <QString>
#include <functional>

class SampleModel;
class MaterialModel;
class Material;
class MaterialItem;
class ParticleLayoutItem;
class ParticleItem;
class ItemWithParticles;
class SessionItem;
class MultiLayer;
class IParticle;
class Particle;
class MultiLayerItem;

//! Builds GUI sample structure from a domain sample structure.

namespace GUI::Transform::FromDomain {

class GUISampleBuilder {
public:
    MultiLayerItem* populateSampleModel(SampleModel* sampleModel, MaterialModel* materialModel,
                                        const MultiLayer& multiLayer, const QString& nodeName = "");

private:
    MaterialItem* createMaterialFromDomain(const Material*);

    void copyParticle(const IParticle* particle,
                      std::function<void(ItemWithParticles*)> addToParent);
    void copyParticle(ParticleItem* particleItem, const Particle* particle);


    QString m_topSampleName;
    SampleModel* m_sampleModel = nullptr;
    MaterialModel* m_materialModel = nullptr;
};

} // namespace GUI::Transform::FromDomain

#endif // BORNAGAIN_GUI_MODEL_FROM_GUISAMPLEBUILDER_H
