//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Instrument/LinkInstrumentManager.h
//! @brief     Defines class LinkInstrumentManager
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_INSTRUMENT_LINKINSTRUMENTMANAGER_H
#define BORNAGAIN_GUI_MODEL_INSTRUMENT_LINKINSTRUMENTMANAGER_H

#include <QList>
#include <QObject>
#include <QStringList>
#include <QVector>

class InstrumentModel;
class RealDataModel;
class InstrumentItem;
class RealDataItem;

//! The LinkInstrumentManager class provides communication between InstrumentModel and
//! RealDataModel. Particularly, it notifies RealDataItem about changes in linked instruments
//! to adjust axes of IntensityDataItem.

class LinkInstrumentManager : public QObject {
    Q_OBJECT

public:
    LinkInstrumentManager(InstrumentModel* instrumentModel, RealDataModel* realDataModel);

    //! Returns true if RealDataItem can be linked to the instrument (same number of bins).
    //! Also offers dialog to adjust instrument to match shape of real data.
    //! quiet defines whether a "not possible" message box is shown if link is not possible. Use
    //! this e.g. for unit tests. The question for adjusting the instrument is not suppressed by
    //! this flag.
    bool canLinkDataToInstrument(const RealDataItem* realDataItem, const QString& identifier,
                                 QWidget* parent);

private:
    void onInstrumentChanged(const InstrumentItem* instrument);
    void onInstrumentAddedOrRemoved();

private:
    InstrumentModel* m_instrumentModel;
    RealDataModel* m_realDataModel;
};

#endif // BORNAGAIN_GUI_MODEL_INSTRUMENT_LINKINSTRUMENTMANAGER_H
