//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/SampleValidator.h
//! @brief     Defines class SampleValidator
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_SAMPLEVALIDATOR_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_SAMPLEVALIDATOR_H

#include <QModelIndex>

class SessionItem;
class MultiLayerItem;
class ParticleCompositionItem;
class ParticleCoreShellItem;
class ParticleLayoutItem;
class LayerItem;

//! Validates SampleModel for MultiLayerItem suitable for simulation
class SampleValidator {
public:
    bool isValidMultiLayer(const MultiLayerItem* multilayer);

    QString getValidationMessage() const { return m_messages; }

private:
    void iterateItems(const SessionItem* parentItem);
    void validateItem(const SessionItem* item);

    void validateInterferences(const LayerItem* item);

    //! Adds this message to the report.
    void addMessage(const QString& m);

    QString m_messages;
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_SAMPLEVALIDATOR_H
