//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/DetectorEditor.h
//! @brief     Defines class DetectorEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_INSTRUMENT_DETECTOREDITOR_H
#define BORNAGAIN_GUI_VIEW_INSTRUMENT_DETECTOREDITOR_H

#include <QWidget>

class QComboBox;
class QStackedLayout;
class Instrument2DItem;
class RectangularDetectorEditor;
class SphericalDetectorEditor;


//! Contains stack of detector editors and the logic to show proper editor for certain type
//! of detector item (SphericalDetectorEditor or RectangularDetectorEditor).

class DetectorEditor : public QWidget {
    Q_OBJECT

public:
    DetectorEditor(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(Instrument2DItem* item);
    void clearItem();

private slots:
    void onDetectorSelected(int index);

private:
    QComboBox* m_combo;
    QStackedLayout* m_stack;
    RectangularDetectorEditor* m_rectangularEditor;
    SphericalDetectorEditor* m_sphericalEditor;
    Instrument2DItem* m_item;
};

#endif // BORNAGAIN_GUI_VIEW_INSTRUMENT_DETECTOREDITOR_H
