//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Job/JobView.h
//! @brief     Defines class JobView
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_JOB_JOBVIEW_H
#define BORNAGAIN_GUI_VIEW_JOB_JOBVIEW_H

#include <QActionGroup>
#include <QMainWindow>

class MainWindow;
class JobView;
class JobSelectorWidget;
class JobRealTimeWidget;
class FitActivityPanel;
class JobMessagePanel;
class JobProgressAssistant;
class JobResultsPresenter;
class JobItem;
class DocksController;
class ProjectDocument;
enum class JobViewActivity;

//! The JobView class is a main view to show list of jobs, job results and widgets for real time
//! and fitting activities.

class JobView : public QMainWindow {
    Q_OBJECT

public:
    JobView(MainWindow* mainWindow, ProjectDocument* document);

    void fillViewMenu(QMenu* menu);

signals:
    void focusRequest(int);
    void activityChanged(JobViewActivity activity);

public slots:
    void onFocusRequest(JobItem* jobItem);
    void setActivity(JobViewActivity activity);
    void onSelectionChanged(const QVector<JobItem*>& jobs);

private:
    void createSubWindows();
    void createActions();
    void connectSignals();
    void connectActivityRelated();
    void connectJobRelated();

    void setAppropriateActivityForJob(JobItem* jobItem);
    void resetLayout();

    DocksController* m_docks;
    JobProgressAssistant* m_progressAssistant;
    JobItem* m_currentItem;

    JobSelectorWidget* m_jobSelector = nullptr;
    JobResultsPresenter* m_jobResultsPresenter = nullptr;
    JobRealTimeWidget* m_jobRealTimeWidget = nullptr;
    FitActivityPanel* m_fitActivityPanel = nullptr;
    JobMessagePanel* m_jobMessagePanel = nullptr;

    QActionGroup m_activityActions;
    ProjectDocument* m_document;
};

#endif // BORNAGAIN_GUI_VIEW_JOB_JOBVIEW_H
