//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/DWBA/IDWBA.h
//! @brief     Defines class IDWBA.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_RESAMPLE_DWBA_IDWBA_H
#define BORNAGAIN_RESAMPLE_DWBA_IDWBA_H

#include <memory>
#include <optional>

class DiffuseElement; // used by all children
class IFlux;          // used by all children
class IFormFactor;
class WavevectorInfo; // used by all children


//! Abstract base class for form factor evaluations.
//!
//! Wraps an IFormFactor, and provides functions evaluate or evaluatePol.
//! Inherited by IComputeScalar and IComputePol.

class IDWBA {
public:
    virtual ~IDWBA();

    const IFormFactor& ff() const { return *m_ff; }
    size_t iLayer() const;

protected:
    IDWBA(const IFormFactor& ff, size_t i_layer);
    IDWBA(const IFormFactor& ff);

    const std::unique_ptr<const IFormFactor> m_ff;
    const std::optional<size_t> m_i_layer;
};

#endif // BORNAGAIN_RESAMPLE_DWBA_IDWBA_H
#endif // USER_API
