//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/HardParticle/FormFactorIcosahedron.h
//! @brief     Defines class FormFactorIcosahedron.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_HARDPARTICLE_FORMFACTORICOSAHEDRON_H
#define BORNAGAIN_SAMPLE_HARDPARTICLE_FORMFACTORICOSAHEDRON_H

#include "Sample/HardParticle/IFormFactorPolyhedron.h"

//! A regular icosahedron.
//! @ingroup hardParticle

class FormFactorIcosahedron : public IFormFactorPolyhedron {
public:
    inline static const std::string class_name = "FormFactorIcosahedron";
    std::string className() const final { return class_name; }

    FormFactorIcosahedron(std::vector<double> P);
    FormFactorIcosahedron(double edge);

    FormFactorIcosahedron* clone() const override { return new FormFactorIcosahedron(m_edge); }

    double getEdge() const { return m_edge; }

private:
    static const ff::PolyhedralTopology topology;
    const double& m_edge;
};

#endif // BORNAGAIN_SAMPLE_HARDPARTICLE_FORMFACTORICOSAHEDRON_H
