//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/FormFactorDecoratorPositionFactor.cpp
//! @brief     Implements class FormFactorDecoratorPositionFactor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Scattering/FormFactorDecoratorPositionFactor.h"
#include "Base/Vector/WavevectorInfo.h"
#include "Sample/Scattering/Rotations.h"

FormFactorDecoratorPositionFactor::FormFactorDecoratorPositionFactor(const IFormFactor& ff,
                                                                     const R3& position)
    : IFormFactorDecorator(ff), m_position(position)
{
}

double FormFactorDecoratorPositionFactor::bottomZ(const IRotation* rotation) const
{
    return m_ff->bottomZ(rotation) +
        ( rotation ? rotation->transformed(m_position) : m_position ).z();
}

double FormFactorDecoratorPositionFactor::topZ(const IRotation* rotation) const
{
    return m_ff->topZ(rotation) +
        ( rotation ? rotation->transformed(m_position) : m_position ).z();
}

complex_t FormFactorDecoratorPositionFactor::theFF(const WavevectorInfo& wavevectors) const
{
    return getPositionFactor(wavevectors) * m_ff->theFF(wavevectors);
}

Eigen::Matrix2cd
FormFactorDecoratorPositionFactor::thePolFF(const WavevectorInfo& wavevectors) const
{
    return getPositionFactor(wavevectors) * m_ff->thePolFF(wavevectors);
}

complex_t
FormFactorDecoratorPositionFactor::getPositionFactor(const WavevectorInfo& wavevectors) const
{
    C3 q = wavevectors.getQ();
    return exp_I(m_position.dot(q));
}
