//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/SoftParticle/FormFactorSphereGaussianRadius.h
//! @brief     Defines and implements class FormFactorSphereGaussianRadius.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_SOFTPARTICLE_FORMFACTORSPHEREGAUSSIANRADIUS_H
#define BORNAGAIN_SAMPLE_SOFTPARTICLE_FORMFACTORSPHEREGAUSSIANRADIUS_H

#include "Sample/Scattering/IBornFF.h"
#include <memory>

//! A sphere with gaussian radius distribution.
//! @ingroup softParticle

class FormFactorSphereGaussianRadius : public IBornFF {
public:
    inline static const std::string class_name = "FormFactorSphereGaussianRadius";
    std::string className() const final { return class_name; }

    FormFactorSphereGaussianRadius(std::vector<double> P);
    FormFactorSphereGaussianRadius(double mean, double sigma);

    FormFactorSphereGaussianRadius* clone() const override
    {
        return new FormFactorSphereGaussianRadius(m_mean, m_sigma);
    }

    double radialExtension() const override { return m_mean; }

    complex_t formfactor(C3 q) const override;

private:
    double calculateMeanR3() const;

    const double& m_mean; //!< This is the mean radius
    const double& m_sigma;
    double m_mean_r3; //!< This is the radius that gives the mean volume
};

#endif // BORNAGAIN_SAMPLE_SOFTPARTICLE_FORMFACTORSPHEREGAUSSIANRADIUS_H
