#include "Base/Const/Units.h"
#include "GUI/Model/Sample/FormFactorItems.h"
#include "GUI/Model/Sample/InterferenceItems.h"
#include "Sample/Aggregate/Interferences.h"
#include "Sample/HardParticle/HardParticles.h"
#include "Tests/GTestWrapper/google_test.h"

class TestGUICoreObjectCorrespondence : public ::testing::Test {
public:
    void FFCorrespondence(const SessionItem& gui_object, const INode& core_object)
    {
        EXPECT_EQ("FormFactor" + gui_object.displayName().toStdString(), core_object.className());
    }
    void OtherCorrespondence(const SessionItem& gui_object, const INode& core_object)
    {
        EXPECT_EQ(gui_object.displayName().toStdString(), core_object.className());
    }
};

TEST_F(TestGUICoreObjectCorrespondence, AnisoPyramid)
{
    AnisoPyramidItem gui_anisopyramid;
    FormFactorAnisoPyramid core_anisopyramid(1.0, 2.0, 0.1, 45.0 * Units::deg);
    FFCorrespondence(gui_anisopyramid, core_anisopyramid);
}

TEST_F(TestGUICoreObjectCorrespondence, Box)
{
    BoxItem gui_box;
    FormFactorBox core_box(1.0, 1.5, 3.0);
    FFCorrespondence(gui_box, core_box);
}

TEST_F(TestGUICoreObjectCorrespondence, Cone)
{
    ConeItem gui_cone;
    FormFactorCone core_cone(1.0, 0.2, 45.0 * Units::deg);
    FFCorrespondence(gui_cone, core_cone);
}

TEST_F(TestGUICoreObjectCorrespondence, Cone6)
{
    Cone6Item gui_cone6;
    FormFactorCone6 core_cone6(1.0, 0.2, 45.0 * Units::deg);
    FFCorrespondence(gui_cone6, core_cone6);
}

TEST_F(TestGUICoreObjectCorrespondence, Cuboctahedron)
{
    CuboctahedronItem gui_cuboctahedron;
    FormFactorCuboctahedron core_cuboctahedron(1.0, 0.4, 1.0, 45.0 * Units::deg);
    FFCorrespondence(gui_cuboctahedron, core_cuboctahedron);
}

TEST_F(TestGUICoreObjectCorrespondence, Cylinder)
{
    CylinderItem gui_cylinder;
    FormFactorCylinder core_cylinder(1.0, 3.0);
    FFCorrespondence(gui_cylinder, core_cylinder);
}

TEST_F(TestGUICoreObjectCorrespondence, Dodecahedron)
{
    DodecahedronItem gui_dodecahedron;
    FormFactorDodecahedron core_dodecahedron(3.0);
    FFCorrespondence(gui_dodecahedron, core_dodecahedron);
}

TEST_F(TestGUICoreObjectCorrespondence, EllipsoidalCylinder)
{
    EllipsoidalCylinderItem gui_ellcylinder;
    FormFactorEllipsoidalCylinder core_ellcylinder(2.0, 1.0, 1.0);
    FFCorrespondence(gui_ellcylinder, core_ellcylinder);
}

TEST_F(TestGUICoreObjectCorrespondence, FullSphere)
{
    FullSphereItem gui_sphere;
    FormFactorFullSphere core_sphere(1.0);
    FFCorrespondence(gui_sphere, core_sphere);
}

TEST_F(TestGUICoreObjectCorrespondence, FullSpheroid)
{
    FullSpheroidItem gui_spheroid;
    FormFactorFullSpheroid core_spheroid(1.0, 2.0);
    FFCorrespondence(gui_spheroid, core_spheroid);
}

TEST_F(TestGUICoreObjectCorrespondence, HemiEllipsoid)
{
    HemiEllipsoidItem gui_hemiellipsoid;
    FormFactorHemiEllipsoid core_hemiellipsoid(2.0, 1.0, 0.5);
    FFCorrespondence(gui_hemiellipsoid, core_hemiellipsoid);
}

TEST_F(TestGUICoreObjectCorrespondence, Icosahedron)
{
    IcosahedronItem gui_icosahedron;
    FormFactorIcosahedron core_icosahedron(8.0);
    FFCorrespondence(gui_icosahedron, core_icosahedron);
}

TEST_F(TestGUICoreObjectCorrespondence, Prism3)
{
    Prism3Item gui_prism3;
    FormFactorPrism3 core_prism3(1.0, 2.0);
    FFCorrespondence(gui_prism3, core_prism3);
}

TEST_F(TestGUICoreObjectCorrespondence, Prism6)
{
    Prism6Item gui_prism6;
    FormFactorPrism6 core_prism6(1.0, 2.0);
    FFCorrespondence(gui_prism6, core_prism6);
}

TEST_F(TestGUICoreObjectCorrespondence, Pyramid)
{
    PyramidItem gui_pyramid;
    FormFactorPyramid core_pyramid(1.0, 0.2, 45.0 * Units::deg);
    FFCorrespondence(gui_pyramid, core_pyramid);
}

TEST_F(TestGUICoreObjectCorrespondence, CosineRippleBox)
{
    CosineRippleBoxItem gui_ripple1;
    FormFactorCosineRippleBox core_ripple1(10.0, 2.0, 1.0);
    FFCorrespondence(gui_ripple1, core_ripple1);
}

TEST_F(TestGUICoreObjectCorrespondence, SawtoothRippleBox)
{
    SawtoothRippleBoxItem gui_ripple2;
    FormFactorSawtoothRippleBox core_ripple2(10.0, 2.0, 1.0, 0.1);
    FFCorrespondence(gui_ripple2, core_ripple2);
}

TEST_F(TestGUICoreObjectCorrespondence, Tetrahedron)
{
    TetrahedronItem gui_tetrahedron;
    FormFactorTetrahedron core_tetrahedron(1.0, 0.1, 45.0 * Units::deg);
    FFCorrespondence(gui_tetrahedron, core_tetrahedron);
}

TEST_F(TestGUICoreObjectCorrespondence, TruncatedCube)
{
    TruncatedCubeItem gui_trunccube;
    FormFactorTruncatedCube core_trunccube(2.0, 0.2);
    FFCorrespondence(gui_trunccube, core_trunccube);
}

TEST_F(TestGUICoreObjectCorrespondence, TruncatedSphere)
{
    TruncatedSphereItem gui_truncsphere;
    FormFactorTruncatedSphere core_truncsphere(1.0, 0.5, 0);
    FFCorrespondence(gui_truncsphere, core_truncsphere);
}

TEST_F(TestGUICoreObjectCorrespondence, TruncatedSpheroid)
{
    TruncatedSpheroidItem gui_truncspheroid;
    FormFactorTruncatedSpheroid core_truncspheroid(1.0, 1.5, 1.5, 0);
    FFCorrespondence(gui_truncspheroid, core_truncspheroid);
}

TEST_F(TestGUICoreObjectCorrespondence, RadialParacrystal)
{
    InterferenceRadialParaCrystalItem gui_radialparacrystal;
    InterferenceRadialParaCrystal core_radialparacrystal(10.0, 1e-6);
    OtherCorrespondence(gui_radialparacrystal, core_radialparacrystal);
}

TEST_F(TestGUICoreObjectCorrespondence, 1DLattice)
{
    Interference1DLatticeItem gui_1d_lattice;
    Interference1DLattice core_1d_lattice(20.0, 0.0);
    OtherCorrespondence(gui_1d_lattice, core_1d_lattice);
}
