#include "Param/Node/INode.h"
#include "Fit/Param/RealLimits.h"
#include "Param/Node/NodeUtils.h"
#include "Tests/GTestWrapper/google_test.h"
#include <memory>
#include <stdexcept>

namespace {

const std::string test_class_name = "TestClass";
const std::string another_test_class_name = "AnotherTestClass";

} // namespace

class INodeTest : public ::testing::Test {
public:
    class TestClass : public INode {
    public:
        inline static const std::string class_name = "TestClass";
        std::string className() const final { return class_name; }

        TestClass() = default;

        ~TestClass() override
        {
            for (auto* child : m_nodes)
                delete child;
        }

        void appendChild(INode* node) { m_nodes.push_back(node); }

        std::vector<const INode*> nodeChildren() const override
        {
            return {m_nodes.begin(), m_nodes.end()};
        }

        std::vector<INode*> m_nodes;
        double m_parameter1;

        friend class INodeTest_fullQualifiedParameters_Test;
        friend class INodeTest_createChildParameterTree_Test;
    };
};

TEST_F(INodeTest, initialState)
{
    INodeTest::TestClass node;
    EXPECT_EQ(node.nodeChildren().size(), 0u);
}

TEST_F(INodeTest, appendChild)
{
    INodeTest::TestClass node;

    auto* child0 = new INodeTest::TestClass();

    // inserting single child
    node.appendChild(child0);

    // accessing vector of children
    auto children = node.nodeChildren();
    EXPECT_EQ(children.size(), 1u);
    EXPECT_TRUE(std::find(children.begin(), children.end(), child0) != children.end());
}

TEST_F(INodeTest, checkLimits)
{
    // this checks only the function if RealLimits.check() works in principle.
    // More tests regarding RealLimits should be done in dedicated RealLimits unit tests.
    EXPECT_NO_THROW(RealLimits::nonnegative().check("A", 0));
    EXPECT_THROW(RealLimits::positive().check("A", 0), std::runtime_error);
}
